﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using ComputeFileHashes;

[assembly: SuppressMessage("Microsoft.Design", "CA2210:AssembliesShouldHaveValidStrongNames", Justification = "Sample application")]

namespace ComputeFileHashesCL
{
    /// <summary>
    /// Implements the command-line version of ComputeFileHashes.
    /// </summary>
    [SuppressMessage("Microsoft.Naming", "CA1724:TypeNamesShouldNotMatchNamespaces", Justification = "Deliberate separation of concerns")]
    public static class ComputeFileHashesCL
    {
        /// <summary>
        /// Implements entry point of program.
        /// </summary>
        /// <param name="args">Program arguments.</param>
        [SuppressMessage("Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "ComputeFileHashesCL", Justification = "Program name.")]
        [SuppressMessage("Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "FileOrPattern", Justification = "Parameter name.")]
        [SuppressMessage("Microsoft.Naming", "CA2204:Literals should be spelled correctly", MessageId = "vhd", Justification = "Sample file extension.")]
        [SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters", MessageId = "System.Console.Write(System.String,System.Object)", Justification = "Not translating.")]
        [SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters", MessageId = "System.Console.WriteLine(System.String)", Justification = "Not translating.")]
        [SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters", MessageId = "System.Console.WriteLine(System.String,System.Object)", Justification = "Not translating.")]
        [SuppressMessage("Microsoft.Globalization", "CA1303:Do not pass literals as localized parameters", MessageId = "System.Console.WriteLine(System.String,System.Object,System.Object)", Justification = "Not translating.")]
        public static void Main(string[] args)
        {
            // Validate argument
            if (null == args)
            {
                throw new ArgumentNullException("args");
            }

            // Display syntax for invalid arguments
            if (0 == args.Length)
            {
                Console.WriteLine(
                    "\n" +
                    "ComputeFileHashesCL\n" +
                    "   Computes multiple hashes for the specified file(s).\n" +
                    "   Version " + ComputeFileHashesCore.Version + "\n" +
                    "   http://blogs.msdn.com/b/delay/\n" +
                    "\n" +
                    "Syntax: ComputeFileHashesCL FileOrPattern [FileOrPattern [...]]\n" +
                    "   Each FileOrPattern can specify a single file or a set of files\n" +
                    "\n" +
                    "Examples:\n" +
                    "   ComputeFileHashesCL Image.iso\n" +
                    "   ComputeFileHashesCL *.iso\n" +
                    "   ComputeFileHashesCL *.iso *.vhd");
                return;
            }

            using (var computeFileHashesCore = new ComputeFileHashesCore())
            {
                // Process arguments
                foreach (var arg in args)
                {
                    // Get directory from argument; default to current
                    var path = Path.GetDirectoryName(arg);
                    if (string.IsNullOrEmpty(path))
                    {
                        path = ".";
                    }
                    // Process matching files
                    foreach (var fileInfo in new DirectoryInfo(path).GetFiles(Path.GetFileName(arg)))
                    {
                        // Display file name
                        Console.WriteLine();
                        Console.WriteLine("{0}", fileInfo.FullName);

                        try
                        {
                            // Process the file
                            computeFileHashesCore.ProcessFile(fileInfo, (percent) => { Console.Write("{0:0.0}%\r", percent); return true; });

                            // Output computed hashes
                            foreach (var hash in computeFileHashesCore.Hashes)
                            {
                                Console.WriteLine("{0}: {1}", hash.Key, ComputeFileHashesCore.ConvertValueToString(hash.Value.Hash));
                            }
                        }
                        catch (Exception e)
                        {
                            // Output error information
                            Console.WriteLine("ERROR: {0}", e.Message);
                        }
                    }
                }
            }
        }
    }
}
