﻿using System;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using ComputeFileHashes;

[assembly: SuppressMessage("Microsoft.Design", "CA2210:AssembliesShouldHaveValidStrongNames", Justification = "Sample application")]
[assembly: SuppressMessage("Microsoft.Design", "CA1020:AvoidNamespacesWithFewTypes", Scope = "namespace", Target = "ComputeFileHashes", Justification = "Sample application")]
[assembly: SuppressMessage("Microsoft.Design", "CA1020:AvoidNamespacesWithFewTypes", Scope = "namespace", Target = "ComputeFileHashesSL", Justification = "Sample application")]

namespace ComputeFileHashesSL
{
    /// <summary>
    /// Handles the UI for ComputeFileHashesSL.
    /// </summary>
    public partial class Page : UserControl
    {
        // Implementation of UI functionality
        private readonly ComputeFileHashesUI _computeFileHashesUI;

        /// <summary>
        /// Initializes a new instance.
        /// </summary>
        public Page()
        {
            InitializeComponent();

            // Initialize variables
            _computeFileHashesUI = new ComputeFileHashesUI(Dispatcher);
            _computeFileHashesUI.Initialize(FileDisplay, VersionBlock);

            // Handle closing event to stop UI
            Application.Current.Exit += delegate { _computeFileHashesUI.Stop(); };
        }

        /// <summary>
        /// Handles clicks on the "Add Files" button.
        /// </summary>
        /// <param name="sender">Sending object.</param>
        /// <param name="e">Event arguments.</param>
        [SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode", Justification = "Called from XAML-based event handler.")]
        private void AddFiles_Click(object sender, RoutedEventArgs e)
        {
            // Pass click on to handler
            _computeFileHashesUI.ShowOpenFileDialog();
        }

        /// <summary>
        /// Handles the Drop event.
        /// </summary>
        /// <param name="e">Event arguments.</param>
        protected override void OnDrop(DragEventArgs e)
        {
            // Validate argument
            if (null == e)
            {
                throw new ArgumentNullException("e");
            }

            // Call base implementation
            base.OnDrop(e);

            // Get drop data as a FileInfo[]
            var fileDrop = e.Data.GetData(DataFormats.FileDrop) as FileInfo[];
            if (null != fileDrop)
            {
                foreach (var fileOrDirectory in fileDrop)
                {
                    // Queue files for hashing
                    if (fileOrDirectory.Exists)
                    {
                        _computeFileHashesUI.QueueFile(new HashFileInfo(fileOrDirectory, Dispatcher));
                    }
                    else
                    {
                        // Directory; can't access it at normal trust level
                    }
                }
            }
        }
    }
}
