﻿using System;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.IO;
using System.Windows;
using System.Windows.Documents;
using System.Windows.Navigation;
using ComputeFileHashes;

[assembly: SuppressMessage("Microsoft.Design", "CA2210:AssembliesShouldHaveValidStrongNames", Justification = "Sample application")]
[assembly: SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "WPF", Justification = "Casing is consistent with acronym")]
[assembly: SuppressMessage("Microsoft.Naming", "CA1709:IdentifiersShouldBeCasedCorrectly", MessageId = "WPF", Scope = "namespace", Target = "ComputeFileHashesWPF", Justification = "Casing is consistent with acronym")]

namespace ComputeFileHashesWPF
{
    /// <summary>
    /// Handles the UI for ComputeFileHashesWPF.
    /// </summary>
    public partial class Window1 : Window
    {
        // Implementation of UI functionality
        private readonly ComputeFileHashesUI _computeFileHashesUI;

        /// <summary>
        /// Initializes a new instance.
        /// </summary>
        public Window1()
        {
            InitializeComponent();

            // Initialize variables
            _computeFileHashesUI = new ComputeFileHashesUI(Dispatcher);
            _computeFileHashesUI.Initialize(FileDisplay, VersionBlock);

            // Handle closing event to stop UI
            Closing += delegate { _computeFileHashesUI.Stop(); };
        }

        /// <summary>
        /// Handles clicks on the "Add Files" button.
        /// </summary>
        /// <param name="sender">Sending object.</param>
        /// <param name="e">Event arguments.</param>
        private void AddFiles_Click(object sender, RoutedEventArgs e)
        {
            // Pass click on to handler
            _computeFileHashesUI.ShowOpenFileDialog();
        }

        /// <summary>
        /// Handles clicks on the blog URI
        /// </summary>
        /// <param name="sender">Sending object.</param>
        /// <param name="e">Event arguments.</param>
        private void Hyperlink_RequestNavigate(object sender, RequestNavigateEventArgs e)
        {
            // Start an instance of the URI (in a browser window)
            Process.Start(((Hyperlink)sender).NavigateUri.ToString());
        }

        /// <summary>
        /// Handles the Drop event.
        /// </summary>
        /// <param name="e">Event arguments.</param>
        protected override void OnDrop(DragEventArgs e)
        {
            // Validate argument
            if (null == e)
            {
                throw new ArgumentNullException("e");
            }

            // Call base implementation
            base.OnDrop(e);

            // Get drop data as a string[] (file names)
            var fileDrop = e.Data.GetData(DataFormats.FileDrop) as string[];
            if (null != fileDrop)
            {
                foreach (var fileOrDirectory in fileDrop)
                {
                    // Get file or files in directory
                    var files = Directory.Exists(fileOrDirectory) ?
                        new DirectoryInfo(fileOrDirectory).GetFiles() :
                        new[] { new FileInfo(fileOrDirectory) };

                    // Queue files for hashing
                    foreach (var file in files)
                    {
                        _computeFileHashesUI.QueueFile(new HashFileInfo(file, Dispatcher));
                    }
                }
            }
        }
    }
}
